@@ -40,6 +40,18 @@ static const op_usage opcodes[] = {
 	/*  9 */	{ "BW_OR", ALL_USED },
 	/*  10 */	{ "BW_AND", ALL_USED },
 	/*  11 */	{ "BW_XOR", ALL_USED },
+#if PHP_VERSION_ID >= 70400
+	/*  12 */	{ "POW", ALL_USED },
+	/*  13 */	{ "BW_NOT", RES_USED | OP1_USED },
+	/*  14 */	{ "BOOL_NOT", RES_USED | OP1_USED },
+	/*  15 */	{ "BOOL_XOR", ALL_USED },
+	/*  16 */	{ "IS_IDENTICAL", ALL_USED },
+	/*  17 */	{ "IS_NOT_IDENTICAL", ALL_USED },
+	/*  18 */	{ "IS_EQUAL", ALL_USED },
+	/*  19 */	{ "IS_NOT_EQUAL", ALL_USED },
+	/*  20 */	{ "IS_SMALLER", ALL_USED },
+	/*  21 */	{ "IS_SMALLER_OR_EQUAL", ALL_USED },
+#else
 	/*  12 */	{ "BW_NOT", RES_USED | OP1_USED },
 	/*  13 */	{ "BOOL_NOT", RES_USED | OP1_USED },
 	/*  14 */	{ "BOOL_XOR", ALL_USED },
@@ -50,6 +62,8 @@ static const op_usage opcodes[] = {
 	/*  19 */	{ "IS_SMALLER", ALL_USED },
 	/*  20 */	{ "IS_SMALLER_OR_EQUAL", ALL_USED },
 	/*  21 */	{ "CAST", ALL_USED | EXT_VAL },
+#endif
+
 #if PHP_VERSION_ID >= 70400
 	/*  22 */	{ "ASSIGN", ALL_USED },
 	/*  23 */	{ "ASSIGN_DIM", ALL_USED },
@@ -110,7 +124,11 @@ static const op_usage opcodes[] = {
 #else
 	/*  49 */	{ "CHECK_VAR", OP1_USED },
 	/*  50 */	{ "SEND_VAR_NO_REF_EX", ALL_USED },
-	/*  51 */	{ "MAKE_REF", RES_USED| OP1_USED },
+#if PHP_VERSION_ID >= 70400
+	/*  51 */	{ "CAST", ALL_USED | EXT_VAL },
+#else
+	/*  51 */	{ "MAKE_REF", RES_USED| OP1_USED},
+#endif
 #endif
 	/*  52 */	{ "BOOL", RES_USED | OP1_USED },
 	/*  53 */	{ "FAST_CONCAT", ALL_USED },
@@ -217,8 +235,13 @@ static const op_usage opcodes[] = {
 #endif
 	/*  137 */	{ "OP_DATA", ALL_USED },
 	/*  138 */	{ "INSTANCEOF", ALL_USED },
+#if PHP_VERSION_ID >= 70400
+	/*  139 */	{ "GENERATOR_CREATE", ALL_USED },
+	/*  140 */	{ "MAKE_REF", RES_USED| OP1_USED },
+#else
 	/*  139 */	{ "DECLARE_CLASS", ALL_USED },
 	/*  140 */	{ "DECLARE_INHERITED_CLASS", ALL_USED },
+#endif
 	/*  141 */	{ "DECLARE_FUNCTION", ALL_USED },
 #if PHP_VERSION_ID >= 70400
 	/*  142 */	{ "DECLARE_LAMBDA_FUNCTION", OP1_USED },
@@ -226,7 +249,11 @@ static const op_usage opcodes[] = {
 	/*  142 */	{ "RAISE_ABSTRACT_ERROR", ALL_USED },
 #endif
 	/*  143 */	{ "DECLARE_CONST", OP1_USED | OP2_USED },
+#if PHP_VERSION_ID >= 70400
+	/*  144 */	{ "DECLARE_CLASS", ALL_USED },
+#else
 	/*  144 */	{ "ADD_INTERFACE", ALL_USED },
+#endif
 #if PHP_VERSION_ID >= 70400
 	/*  145 */	{ "DECLARE_CLASS_DELAYED", ALL_USED },
     /*  146 */	{ "DECLARE_ANON_CLASS", OP2_USED | RES_USED | RES_CLASS },
@@ -307,6 +334,13 @@ static const op_usage opcodes[] = {
 	/*  193 */	{ "GET_TYPE", ALL_USED },
 # if PHP_VERSION_ID >= 70400
 	/*  194 */	{ "ARRAY_KEY_EXISTS", ALL_USED },
+#  if PHP_VERSION_ID >= 80000
+	/*  195 */	{ "MATCH", ALL_USED | OP2_JMP_ARRAY },
+	/*  196 */	{ "CASE_STRICT", ALL_USED },
+	/*  197 */	{ "MATCH_ERROR", ALL_USED },
+	/*  198 */	{ "JMP_NULL", ALL_USED },
+	/*  199 */	{ "CHECK_UNDEF_ARGS", ALL_USED },
+#  endif
 # else
 	/*  194 */	{ "FUNC_NUM_ARGS", ALL_USED },
 	/*  195 */	{ "FUNC_GET_ARGS", ALL_USED },
@@ -395,11 +429,6 @@ static inline int vld_dump_zval_reference(ZVAL_VALUE_TYPE value)
 	return vld_printf (stderr, "<reference>");
 }
 
-static inline int vld_dump_zval_callable(ZVAL_VALUE_TYPE value)
-{
-	return vld_printf (stderr, "<callable>");
-}
-
 static inline int vld_dump_zval_indirect(ZVAL_VALUE_TYPE value)
 {
 	return vld_printf (stderr, "<indirect>");
@@ -428,7 +457,6 @@ int vld_dump_zval (zval val)
 		case IS_FALSE:          return vld_dump_zval_false (val.value);
 		case IS_TRUE:           return vld_dump_zval_true (val.value);
 		case IS_REFERENCE:      return vld_dump_zval_reference (val.value);
-		case IS_CALLABLE:       return vld_dump_zval_callable (val.value);
 		case IS_INDIRECT:       return vld_dump_zval_indirect (val.value);
 		case IS_PTR:            return vld_dump_zval_ptr (val.value);
 	}
@@ -436,7 +464,7 @@ int vld_dump_zval (zval val)
 }
 
 
-int vld_dump_znode (int *print_sep, unsigned int node_type, VLD_ZNODE node, unsigned int base_address, zend_op_array *op_array, int opline TSRMLS_DC)
+int vld_dump_znode (int *print_sep, unsigned int node_type, VLD_ZNODE node, unsigned int base_address, zend_op_array *op_array, int opline)
 {
 	int len = 0;
 
@@ -500,11 +528,14 @@ int vld_dump_znode (int *print_sep, unsigned int node_type, VLD_ZNODE node, unsi
 			myht = Z_ARRVAL_P(array_value);
 
 			len += vld_printf (stderr, "[ ");
+			ZVAL_VALUE_STRING_TYPE *new_str;
 			ZEND_HASH_FOREACH_KEY_VAL_IND(myht, num, key, val) {
 				if (key == NULL) {
 					len += vld_printf (stderr, "%d:->%d, ", num, opline + (val->value.lval / sizeof(zend_op)));
 				} else {
-					len += vld_printf (stderr, "'%s':->%d, ", ZSTRING_VALUE(key), opline + (val->value.lval / sizeof(zend_op)));
+					new_str = php_url_encode(ZSTRING_VALUE(key), key->len PHP_URLENCODE_NEW_LEN(new_len));
+					len += vld_printf (stderr, "'%s':->%d, ", ZSTRING_VALUE(new_str), opline + (val->value.lval / sizeof(zend_op)));
+					efree(new_str);
 				}
 			} ZEND_HASH_FOREACH_END();
 
@@ -557,12 +588,17 @@ static unsigned int vld_get_special_flags(const zend_op *op, unsigned int base_a
 			break;
 
 		case ZEND_FETCH_CLASS:
-			flags = EXT_VAL|RES_USED|OP2_USED|RES_CLASS;
+			flags = EXT_VAL|RES_USED|OP2_USED;
 			break;
 
 		case ZEND_NEW:
-			flags = RES_USED|OP1_USED;
-			flags |= OP1_CLASS;
+			flags = RES_USED;
+			if (op->VLD_TYPE(op1) != IS_UNUSED) {
+				flags |= OP1_USED;
+				if (op->VLD_TYPE(op1) == IS_CONST) {
+					flags |= OP1_CLASS;
+				}
+			}
 			break;
 
 		case ZEND_BRK:
@@ -627,9 +663,9 @@ static const char *get_assign_operation(uint32_t extended_value)
 }
 #endif
 
-void vld_dump_op(int nr, zend_op * op_ptr, unsigned int base_address, int notdead, int entry, int start, int end, zend_op_array *opa TSRMLS_DC)
+void vld_dump_op(int nr, zend_op * op_ptr, unsigned int base_address, int notdead, int entry, int start, int end, zend_op_array *opa)
 {
-	static uint last_lineno = (uint) -1;
+	static unsigned int last_lineno = (unsigned int) -1;
 	int print_sep = 0, len;
 	const char *fetch_type = "";
 	unsigned int flags, op1_type, op2_type, res_type;
@@ -696,6 +732,28 @@ void vld_dump_op(int nr, zend_op * op_ptr, unsigned int base_address, int notdea
 		fetch_type = get_assign_operation(op.extended_value);
 	}
 #endif
+#if PHP_VERSION_ID >= 70100
+	if (op.opcode == ZEND_NEW/* && op1_type == IS_UNUSED*/) {
+		int ftype = op.op1.num & ZEND_FETCH_CLASS_MASK;
+#else
+	if (op.opcode == ZEND_FETCH_CLASS) {
+		int ftype = op.extended_value & ZEND_FETCH_CLASS_MASK;
+#endif
+		switch (ftype) {
+			case ZEND_FETCH_CLASS_SELF:
+				fetch_type = "self";
+				break;
+			case ZEND_FETCH_CLASS_PARENT:
+				fetch_type = "parent";
+				break;
+			case ZEND_FETCH_CLASS_STATIC:
+				fetch_type = "static";
+				break;
+			case ZEND_FETCH_CLASS_AUTO:
+				fetch_type = "auto";
+				break;
+		}
+	}
 
 	if (flags & OP_FETCH) {
 		switch (op.VLD_EXTENDED_VALUE(op2)) {
@@ -730,9 +788,9 @@ void vld_dump_op(int nr, zend_op * op_ptr, unsigned int base_address, int notdea
 	}
 
 	if (op.lineno == last_lineno) {
-		vld_printf(stderr, "     ");
+		vld_printf(stderr, "      ");
 	} else {
-		vld_printf(stderr, "%4d ", op.lineno);
+		vld_printf(stderr, "%5d ", op.lineno);
 		last_lineno = op.lineno;
 	}
 
@@ -776,7 +834,7 @@ void vld_dump_op(int nr, zend_op * op_ptr, unsigned int base_address, int notdea
 	if ((flags & RES_USED) && !(op.VLD_EXTENDED_VALUE(result) & EXT_TYPE_UNUSED)) {
 #endif
 		VLD_PRINT(3, " RES[ ");
-		len = vld_dump_znode (NULL, res_type, op.result, base_address, opa, nr TSRMLS_CC);
+		len = vld_dump_znode (NULL, res_type, op.result, base_address, opa, nr);
 		VLD_PRINT(3, " ]");
 		if (VLD_G(format)) {
 			if (len==0) {
@@ -791,7 +849,7 @@ void vld_dump_op(int nr, zend_op * op_ptr, unsigned int base_address, int notdea
 
 	if (flags & OP1_USED) {
 		VLD_PRINT(3, " OP1[ ");
-		vld_dump_znode (&print_sep, op1_type, op.op1, base_address, opa, nr TSRMLS_CC);
+		vld_dump_znode (&print_sep, op1_type, op.op1, base_address, opa, nr);
 		VLD_PRINT(3, " ]");
 	}
 	if (flags & OP2_USED) {
@@ -821,7 +879,7 @@ void vld_dump_op(int nr, zend_op * op_ptr, unsigned int base_address, int notdea
 					break;
 			}
 		} else {
-			vld_dump_znode (&print_sep, op2_type, op.op2, base_address, opa, nr TSRMLS_CC);
+			vld_dump_znode (&print_sep, op2_type, op.op2, base_address, opa, nr);
 		}
 		VLD_PRINT(3, " ]");
 	}
@@ -837,15 +895,15 @@ void vld_dump_op(int nr, zend_op * op_ptr, unsigned int base_address, int notdea
 	}
 	if (flags & NOP2_OPNUM) {
 		zend_op next_op = op_ptr[nr+1];
-		vld_dump_znode (&print_sep, VLD_IS_OPNUM, next_op.op2, base_address, opa, nr TSRMLS_CC);
+		vld_dump_znode (&print_sep, VLD_IS_OPNUM, next_op.op2, base_address, opa, nr);
 	}
 	vld_printf (stderr, "\n");
 }
 
-void vld_analyse_oparray(zend_op_array *opa, vld_set *set, vld_branch_info *branch_info TSRMLS_DC);
-void vld_analyse_branch(zend_op_array *opa, unsigned int position, vld_set *set, vld_branch_info *branch_info TSRMLS_DC);
+void vld_analyse_oparray(zend_op_array *opa, vld_set *set, vld_branch_info *branch_info);
+void vld_analyse_branch(zend_op_array *opa, unsigned int position, vld_set *set, vld_branch_info *branch_info);
 
-void vld_dump_oparray(zend_op_array *opa TSRMLS_DC)
+void vld_dump_oparray(zend_op_array *opa)
 {
 	unsigned int i;
 	int          j;
@@ -857,7 +915,7 @@ void vld_dump_oparray(zend_op_array *opa TSRMLS_DC)
 	branch_info = vld_branch_info_create(opa->last);
 
 	if (VLD_G(dump_paths)) {
-		vld_analyse_oparray(opa, set, branch_info TSRMLS_CC);
+		vld_analyse_oparray(opa, set, branch_info);
 	}
 	if (VLD_G(format)) {
 		vld_printf (stderr, "filename:%s%s\n", VLD_G(col_sep), ZSTRING_VALUE(opa->filename));
@@ -880,18 +938,18 @@ void vld_dump_oparray(zend_op_array *opa TSRMLS_DC)
 	if (VLD_G(format)) {
 		vld_printf(stderr, "line%s# *%s%s%sop%sfetch%sext%sreturn%soperands\n",VLD_G(col_sep),VLD_G(col_sep),VLD_G(col_sep),VLD_G(col_sep),VLD_G(col_sep),VLD_G(col_sep),VLD_G(col_sep),VLD_G(col_sep));
 	} else {
-		vld_printf(stderr, "line     #* E I O op                           fetch          ext  return  operands\n");
+		vld_printf(stderr, "line      #* E I O op                           fetch          ext  return  operands\n");
 		vld_printf(stderr, "-------------------------------------------------------------------------------------\n");
 	}
 	for (i = 0; i < opa->last; i++) {
-		vld_dump_op(i, opa->opcodes, base_address, vld_set_in(set, i), vld_set_in(branch_info->entry_points, i), vld_set_in(branch_info->starts, i), vld_set_in(branch_info->ends, i), opa TSRMLS_CC);
+		vld_dump_op(i, opa->opcodes, base_address, vld_set_in(set, i), vld_set_in(branch_info->entry_points, i), vld_set_in(branch_info->starts, i), vld_set_in(branch_info->ends, i), opa);
 	}
 	vld_printf(stderr, "\n");
 
 	if (VLD_G(dump_paths)) {
 		vld_branch_post_process(opa, branch_info);
 		vld_branch_find_paths(branch_info);
-		vld_branch_info_dump(opa, branch_info TSRMLS_CC);
+		vld_branch_info_dump(opa, branch_info);
 	}
 
 	vld_set_free(set);
@@ -987,6 +1045,9 @@ int vld_find_jumps(zend_op_array *opa, unsigned int position, size_t *jump_count
 		opcode.opcode == ZEND_GENERATOR_RETURN ||
 		opcode.opcode == ZEND_EXIT ||
 		opcode.opcode == ZEND_THROW ||
+#if PHP_VERSION_ID >= 80000
+		opcode.opcode == ZEND_MATCH_ERROR ||
+#endif
 		opcode.opcode == ZEND_RETURN
 	) {
 		jumps[0] = VLD_JMP_EXIT;
@@ -994,6 +1055,9 @@ int vld_find_jumps(zend_op_array *opa, unsigned int position, size_t *jump_count
 		return 1;
 #if PHP_VERSION_ID >= 70200
 	} else if (
+# if PHP_VERSION_ID >= 80000
+		opcode.opcode == ZEND_MATCH ||
+# endif
 		opcode.opcode == ZEND_SWITCH_LONG ||
 		opcode.opcode == ZEND_SWITCH_STRING
 	) {
@@ -1001,11 +1065,11 @@ int vld_find_jumps(zend_op_array *opa, unsigned int position, size_t *jump_count
 		HashTable *myht;
 		zval *val;
 
-#if PHP_VERSION_ID >= 70300
+# if PHP_VERSION_ID >= 70300
 		array_value = RT_CONSTANT((opa->opcodes) + position, opcode.op2);
-#else
+# else
 		array_value = RT_CONSTANT_EX(opa->literals, opcode.op2);
-#endif
+# endif
 		myht = Z_ARRVAL_P(array_value);
 
 		/* All 'case' statements */
@@ -1020,9 +1084,13 @@ int vld_find_jumps(zend_op_array *opa, unsigned int position, size_t *jump_count
 		jumps[*jump_count] = position + (opcode.extended_value / sizeof(zend_op));
 		(*jump_count)++;
 
-		/* The 'next' opcode */
-		jumps[*jump_count] = position + 1;
-		(*jump_count)++;
+# if PHP_VERSION_ID >= 80000
+		if (opcode.opcode != ZEND_MATCH) {
+			/* The 'next' opcode */
+			jumps[*jump_count] = position + 1;
+			(*jump_count)++;
+		}
+# endif
 
 		return 1;
 #endif
@@ -1031,14 +1099,14 @@ int vld_find_jumps(zend_op_array *opa, unsigned int position, size_t *jump_count
 	return 0;
 }
 
-void vld_analyse_oparray(zend_op_array *opa, vld_set *set, vld_branch_info *branch_info TSRMLS_DC)
+void vld_analyse_oparray(zend_op_array *opa, vld_set *set, vld_branch_info *branch_info)
 {
 	unsigned int position = 0;
 
 	VLD_PRINT(1, "Finding entry points\n");
 	while (position < opa->last) {
 		if (position == 0) {
-			vld_analyse_branch(opa, position, set, branch_info TSRMLS_CC);
+			vld_analyse_branch(opa, position, set, branch_info);
 			vld_set_add(branch_info->entry_points, position);
 		} else if (opa->opcodes[position].opcode == ZEND_CATCH) {
 			if (VLD_G(format)) {
@@ -1046,7 +1114,7 @@ void vld_analyse_oparray(zend_op_array *opa, vld_set *set, vld_branch_info *bran
 			} else {
 				VLD_PRINT1(1, "Found catch point at position: %d\n", position);
 			}
-			vld_analyse_branch(opa, position, set, branch_info TSRMLS_CC);
+			vld_analyse_branch(opa, position, set, branch_info);
 			vld_set_add(branch_info->entry_points, position);
 		}
 		position++;
@@ -1055,7 +1123,7 @@ void vld_analyse_oparray(zend_op_array *opa, vld_set *set, vld_branch_info *bran
 	branch_info->branches[opa->last-1].start_lineno = opa->opcodes[opa->last-1].lineno;
 }
 
-void vld_analyse_branch(zend_op_array *opa, unsigned int position, vld_set *set, vld_branch_info *branch_info TSRMLS_DC)
+void vld_analyse_branch(zend_op_array *opa, unsigned int position, vld_set *set, vld_branch_info *branch_info)
 {
 	if (VLD_G(format)) {
 		VLD_PRINT2(1, "Branch analysis from position:%s%d\n", VLD_G(col_sep),position);
@@ -1098,7 +1166,7 @@ void vld_analyse_branch(zend_op_array *opa, unsigned int position, vld_set *set,
 				if (jumps[i] == VLD_JMP_EXIT || jumps[i] >= 0) {
 					vld_branch_info_update(branch_info, position, opa->opcodes[position].lineno, i, jumps[i]);
 					if (jumps[i] != VLD_JMP_EXIT) {
-						vld_analyse_branch(opa, jumps[i], set, branch_info TSRMLS_CC);
+						vld_analyse_branch(opa, jumps[i], set, branch_info);
 					}
 				}
 			}
@@ -1106,6 +1174,16 @@ void vld_analyse_branch(zend_op_array *opa, unsigned int position, vld_set *set,
 			break;
 		}
 
+#if PHP_VERSION_ID >= 80000
+		/* See if we have a match_error instruction */
+		if (opa->opcodes[position].opcode == ZEND_MATCH_ERROR) {
+			VLD_PRINT1(1, "Match error found at %d\n", position);
+			vld_set_add(branch_info->ends, position);
+			branch_info->branches[position].start_lineno = opa->opcodes[position].lineno;
+			break;
+		}
+#endif
+
 		/* See if we have a throw instruction */
 		if (opa->opcodes[position].opcode == ZEND_THROW) {
 			VLD_PRINT1(1, "Throw found at %d\n", position);
